// Copyright (C) 2012 MicroStrain, Inc.

/*************************
*	CCListBox
*	CCListBox is inherited from the CListBox object.  The purpose of this
*	is to override the DrawItem function for the console window allowing
*	different items in the console window to contain different colors.
*************************/

#include "stdafx.h"
#include "2400_sdk.h"
#include "CListBox.h"

/*	functions used for writing to the console	*/
#include "console_commands.h"
extern CMapPosToCString strCmd_info;
extern CMapPosToColor crCmd_color;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// CCListBox

IMPLEMENT_DYNAMIC(CCListBox, CListBox)
CCListBox::CCListBox()
{
}

CCListBox::~CCListBox()
{
}


BEGIN_MESSAGE_MAP(CCListBox, CListBox)
END_MESSAGE_MAP()

/*************************
*	DrawItem()
*	DrawItem() is a virtual function to handle the drawing of each item in the
*	console window.  It is used to add colors to each item in the console window.
*************************/
void CCListBox::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct)
{
	/*	if no items exist and there's nothing to draw, just return	*/
	if (lpDrawItemStruct->itemID == (UINT)-1)
		return;

	ASSERT(lpDrawItemStruct->CtlType == ODT_LISTBOX);
	
	//get the text
	CString strText;
	GetText(lpDrawItemStruct->itemID, strText);

	CDC dc;
	dc.Attach(lpDrawItemStruct->hDC);

	// Save these value to restore them when done drawing.
	COLORREF crOldTextColor = dc.GetTextColor();
	COLORREF crOldBkColor = dc.GetBkColor();

	/*	fill the background of the item with the background color	*/
	dc.FillSolidRect(&lpDrawItemStruct->rcItem, crOldBkColor);

	// If this item has the focus, draw a frame around the
	// item's rect with the color defined by COLOR_OUTLINE
	if ((lpDrawItemStruct->itemAction | ODA_FOCUS) &&
	(lpDrawItemStruct->itemState & ODS_FOCUS))
	{
		CBrush br(COLOR_OUTLINE);
		dc.FrameRect(&lpDrawItemStruct->rcItem, &br);
	}
	
	/*	See if there is a color for that item in our list.  If there is
	*	draw the text in that color instead of the default.	*/
	COLORREF crColor;
	if (crCmd_color.Lookup(lpDrawItemStruct->itemID, crColor))
		dc.SetTextColor(crColor);
	// Draw the text.
	dc.DrawText(
		strText,
		&lpDrawItemStruct->rcItem,
		DT_LEFT|DT_SINGLELINE|DT_VCENTER);

	// Reset the background color and the text color back to their
	// original values.
	dc.SetTextColor(crOldTextColor);
	dc.SetBkColor(crOldBkColor);

	dc.Detach();
}
