// Copyright (C) 2012 MicroStrain, Inc.

/*********************
*	streamsetup.h
*	streamsetup.h holds the CStreamSetup class which allows
*	the user to setup the node for a short stream by setting
*	the number of sweeps and the channel mask.
*********************/

#pragma once
#include "afxcmn.h"
#include "afxwin.h"


// CStreamSetup dialog

class CStreamSetup : public CDialog
{
	DECLARE_DYNAMIC(CStreamSetup)

public:
	CStreamSetup(CWnd* pParent = NULL);   // standard constructor
	virtual ~CStreamSetup();
	virtual BOOL OnInitDialog();

	/***********************
	*	OnEnKillfocusEditSweeps()
	*	OnEnKillfocusEditSweeps() performs a range test on the number of
	*	sweeps.  If the focus on the edit control is lost and the number of
	*	sweeps is set below the low end of the range, the number of sweeps
	*	is set to the low end of the range.  Likewise occurs for the high
	*	end of the range.
	***********************/
	afx_msg void OnEnKillfocusEditSweeps();


	afx_msg void OnBnClickedCheckContinuousStreaming();

// Dialog Data
	enum { IDD = IDD_STREAM_SETUP };
public:
	unsigned short m_usChannel_mask;	//	Channel mask
	bool m_bContinuous_streaming;       //  Continuous streaming
	unsigned short m_usSweeps;	//	Number of sweeps.  When writing this value, it must be divided by 100
								//	See the protocol documentation for more information.
protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	DECLARE_MESSAGE_MAP()
	/***********************
	*	OnOK()
	*	OnOK() performs two checks on the data.  It first calculates a channel mask,
	*	verifies that at least one channel is active, and saves that value.  Second,
	*	the application does a quick check for the number of sweeps to be sampled
	*	during streaming.  If this value is above a certain level a warning is displayed
	*	to let the user know there will be a great amount of text written to the console.
	***********************/
	virtual void OnOK();

private:
	CSpinButtonCtrl m_csbcSweeps;	     // number of sweeps spin control
	CButton m_checkContinuousStreaming;  // enable or disable continuous streaming
};
