// Copyright (C) 2012 MicroStrain, Inc.

/*********************
*	console_commands.h
*	console_commands.h holds the declarations for commands operating
*	on the console window of the application's dialog box.
*********************/
#ifndef	CONSOLE_COMMANDS_H
#define	CONSOLE_COMMANDS_H

#include "clistbox.h"
#include "afxtempl.h"

//maps to map an item's position in the list control to detailed text and color
typedef CMap<int, int, CString, LPCTSTR> CMapPosToCString;
typedef CMap<int, int, COLORREF, COLORREF&> CMapPosToColor;

/************************
*	set_console()
*	set_console() sets the static global pcliConsole pointer to the value given
*	by cliConsole.  This value is then used for other functions to print commands
*	to the console.  This function should only be called once at the beginning of
*	the application's execution as the console window isn't destroyed until the
*	program quits.
*	PRE:	cliConsole is a legitimate CCListBox pointer pointing to the application's
*				console window
*	POST:	pcliConsole now points to the application's console window
************************/
void set_console(/* in */ CCListBox *cliConsole);

/************************
*	console_write()
*	console_write() writes a string, lpszMsg, to the console window.
*	PRE:	strMsg is a CString object holding the information to be written
*	POST:	if successful, strMsg has been written to the window and the index
*				of the the string's location in the console has been returned
*			otherwise, -1 has been returned
************************/
int console_write(/* in */ LPCTSTR lpszMsg);

/************************
*	console_write_command()
*	console_write_command() writes an unsigned character array, ucCmd, to the 
*	screen as a hexidecimal command.  i.e. if we want to output a read command
*	it will appear as 0x03 0x00 0x04 0x00 0x02 to read node 4, EEPROM location 2.
*	PRE:	ucCmd is a legitimate unsigned character array of length dwLength.
*				This array is not null terminated
*			dwLength is the length of the unsigned character array
*	POST:	if successful, ucCmd has been written to the console and the index
*				of the the string's location in the console has been returned
*			otherwise, -1 has been returned
************************/
int console_write_command(/* in */ unsigned char *ucCmd,
							/* in */ DWORD dwLength);

#endif
