Public Class Main

    Private Sub Main_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Sets a value indicating whether to catch calls on the wrong thread that access a control's Handle property 
        txtEvents.CheckForIllegalCrossThreadCalls = False
        txtCommandSent.CheckForIllegalCrossThreadCalls = False
        txtResponseReceived.CheckForIllegalCrossThreadCalls = False
        'populate combo
        cmbCommand.Items.Add("Short Ping")
        cmbCommand.Items.Add("Long Ping")
        cmbCommand.Items.Add("Read Node EEPROM")
        cmbCommand.Items.Add("Write Node EEPROM")
        cmbCommand.Items.Add("Initiate Sleep Mode")
        cmbCommand.Items.Add("Stop Node")
        cmbCommand.Items.Add("Arm Node")
        cmbCommand.Items.Add("Trigger Armed Datalogging")
        cmbCommand.Items.Add("Download Page")
        cmbCommand.Items.Add("Erase Session Data")
        cmbCommand.Items.Add("Initiate Real-Time Streaming")
        cmbCommand.Items.Add("Initiate Low Duty Cycle")
        cmbCommand.Items.Add("Initiate Synchronized Sampling")
        cmbCommand.Items.Add("Read Single Sensor")
        cmbCommand.Items.Add("Auto-Balance Channel")
        cmbCommand.Items.Add("Node Discovery")
        cmbCommand.SelectedItem = cmbCommand.Items.Item(0)
        'populate combo
        cmbDecHex.Items.Add("Decimal")
        cmbDecHex.Items.Add("Hex")
        cmbDecHex.SelectedItem = cmbDecHex.Items.Item(0)
        'populate event
        txtEvents.Text = "Sample code demonstrating node commands and responses..."
        'populate event
        txtEvents.Text = txtEvents.Text + vbCrLf + "We strongly recommend that you follow along in the Data Communications Protocol manual as you run each of these sample commands."
    End Sub

    Private Sub btnSendCommand_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSendCommand.Click
        'clear textboxes
        txtEvents.Text = ""
        txtCommandSent.Text = ""
        txtResponseReceived.Text = ""
        'call command function
        If cmbCommand.Text = "Short Ping" Then
            'call function
            Call Short_Ping()
        ElseIf cmbCommand.Text = "Long Ping" Then
            'call function
            Call Long_Ping()
        ElseIf cmbCommand.Text = "Read Node EEPROM" Then
            'call function
            Call Read_Node_EEPROM()
        ElseIf cmbCommand.Text = "Write Node EEPROM" Then
            'call function
            Call Write_Node_EEPROM()
        ElseIf cmbCommand.Text = "Initiate Sleep Mode" Then
            'call function
            Call Initiate_Sleep_Mode()
        ElseIf cmbCommand.Text = "Stop Node" Then
            'call function
            Call Stop_Node()
        ElseIf cmbCommand.Text = "Arm Node" Then
            'call function
            Call Arm_Node()
        ElseIf cmbCommand.Text = "Trigger Armed Datalogging" Then
            'call function
            Call Trigger_Armed_Datalogging()
        ElseIf cmbCommand.Text = "Download Page" Then
            'call function
            Call Download_Page()
        ElseIf cmbCommand.Text = "Erase Session Data" Then
            'call function
            Call Erase_Session_Data()
        ElseIf cmbCommand.Text = "Initiate Real-Time Streaming" Then
            'call function
            Call Initiate_Real_Time_Streaming()
        ElseIf cmbCommand.Text = "Initiate Low Duty Cycle" Then
            'call function
            Call Initiate_Low_Duty_Cycle()
        ElseIf cmbCommand.Text = "Initiate Synchronized Sampling" Then
            'call function
            Call Initiate_Synchronized_Sampling()
        ElseIf cmbCommand.Text = "Read Single Sensor" Then
            'call function
            Call Read_Single_Sensor()
        ElseIf cmbCommand.Text = "Auto-Balance Channel" Then
            'test for channel 1-4 only
            If numChannelNumber.Value > 4 Then
                'call events
                Call EventHandler("Only channels 1 through 4 can be auto-balanced.")
                'exit
                Exit Sub
            End If
            'call function
            Call Auto_Balance_Channel()
        ElseIf cmbCommand.Text = "Node Discovery" Then
            'call function
            Call Node_Discovery()
        End If
    End Sub

    Private Function EventHandler(ByVal MyEvent As String) As Boolean
        'populate event
        txtEvents.Text = txtEvents.Text + vbCrLf + MyEvent
    End Function

    Private Function ParseArray(ByVal CommandOrResponse As String, ByVal arrParse As Byte()) As Boolean
        'dim
        Dim X As Integer, strBytes As String, PadZero As String
        'seed vars
        strBytes = ""
        PadZero = ""
        'decimal or hex display
        If cmbDecHex.Text = "Decimal" Then
            'construct byte string
            For X = 0 To UBound(arrParse)
                strBytes = strBytes + Chr(32) + Str(arrParse(X))
            Next
        ElseIf cmbDecHex.Text = "Hex" Then
            'construct byte string
            For X = 0 To UBound(arrParse)
                'pad zero
                If Len(CStr(Hex(arrParse(X)))) = 1 Then
                    PadZero = "0" + CStr(Hex(arrParse(X)))
                    strBytes = strBytes + Chr(32) + PadZero
                Else
                    strBytes = strBytes + Chr(32) + CStr(Hex(arrParse(X)))
                End If
            Next
        End If
        'populate command sent or response received textbox
        If CommandOrResponse = "Command" Then
            'populate command sent
            txtCommandSent.Text = strBytes
        ElseIf CommandOrResponse = "Response" Then
            'populate response received
            txtResponseReceived.Text = strBytes
        End If
    End Function

    Private Sub SerialPort1_DataReceived(ByVal sender As System.Object, ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) Handles SerialPort1.DataReceived
        'set error handler
        On Error GoTo ErrorHandler
        'set variable
        UpperLimitOfArray = SerialPort1.BytesToRead - 1
        'prep array
        ReDim arrBytes(0 To UpperLimitOfArray)
        'get comm buffer
        SerialPort1.Read(arrBytes, 0, SerialPort1.BytesToRead)
        'close serial port
        SerialPort1.Close()
        'test for no byte return
        If UBound(arrBytes) = -1 Then
            'call events
            Call EventHandler("No bytes returned")
            'exit
            Exit Sub
        End If
        'populate command textbox
        Call ParseArray("Response", arrBytes)
        'test for command
        '---------------------------------------------------------
        If cmbCommand.Text = "Short Ping" Then
            'test for successful response
            If UBound(arrBytes) = 0 And arrBytes(0) = 2 Then
                'call events
                Call EventHandler("Response successful!")
            Else
                'call events
                Call EventHandler("Response failed!")
            End If
            '---------------------------------------------------------
        ElseIf cmbCommand.Text = "Long Ping" Then
            'test for successful response
            If UBound(arrBytes) = 12 And CheckSum(arrBytes) = True Then
                'call events
                Call EventHandler("Response successful!")
                Call EventHandler("LQI: " + Str(arrBytes(9)))
                Call EventHandler("Base Station RSSI: " + Str(CalcRSSI(arrBytes(10))) + " dBm")
            Else
                'call events
                Call EventHandler("Response failed!")
            End If
            '---------------------------------------------------------
        ElseIf cmbCommand.Text = "Read Node EEPROM" Then
            'test for successful response
            If UBound(arrBytes) = 12 And CheckSum(arrBytes) = True Then
                'call events
                Call EventHandler("Response successful!")
                'populate EEPROM value
                numEEPROMValue.Value = CDbl(arrBytes(7) * 256) + CDbl(arrBytes(8))
                Call EventHandler("LQI: " + Str(arrBytes(9)))
                Call EventHandler("Base Station RSSI: " + Str(CalcRSSI(arrBytes(10))) + " dBm")
            Else
                'call events
                Call EventHandler("Response failed!")
            End If
            '---------------------------------------------------------
        ElseIf cmbCommand.Text = "Write Node EEPROM" Then
            'test for successful response
            If UBound(arrBytes) = 12 And CheckSum(arrBytes) = True Then
                'call events
                Call EventHandler("Response successful!")
                Call EventHandler("LQI: " + Str(arrBytes(9)))
                Call EventHandler("Base Station RSSI: " + Str(CalcRSSI(arrBytes(10))) + " dBm")
            Else
                'call events
                Call EventHandler("Response failed!")
            End If
            '---------------------------------------------------------
        ElseIf cmbCommand.Text = "Initiate Sleep Mode" Then
            'There are no response bytes to this command.
            '---------------------------------------------------------
        ElseIf cmbCommand.Text = "Stop Node" Then
            'test for successful response
            If UBound(arrBytes) = 2 And arrBytes(1) = 144 And arrBytes(2) = 1 Then
                'call events
                Call EventHandler("Response successful!")
            Else
                'call events
                Call EventHandler("Response failed!")
            End If
            '---------------------------------------------------------
        ElseIf cmbCommand.Text = "Arm Node" Then
            'test for successful response
            If UBound(arrBytes) = 13 And CheckSum(arrBytes) = True Then
                'call events
                Call EventHandler("Response successful!")
                Call EventHandler("LQI: " + Str(arrBytes(9)))
                Call EventHandler("Base Station RSSI: " + Str(CalcRSSI(arrBytes(10))) + " dBm")
            Else
                'call events
                Call EventHandler("Response failed!")
            End If
            '---------------------------------------------------------
        ElseIf cmbCommand.Text = "Trigger Armed Datalogging" Then
            'There are no response bytes to this command.
            '---------------------------------------------------------
        ElseIf cmbCommand.Text = "Download Page" Then
            'test for successful response
            If UBound(arrBytes) = 266 And SimpleCheckSum(arrBytes) = True Then
                'call events
                Call EventHandler("Response successful!")
                'call events
                Call EventHandler(MakeDataPoints(arrBytes))
            Else
                'call events
                Call EventHandler("Response failed!")
                ''End If
            End If
            '---------------------------------------------------------
        ElseIf cmbCommand.Text = "Erase Session Data" Then
            'test for successful response
            If UBound(arrBytes) = 0 And arrBytes(0) = 6 Then
                'call events
                Call EventHandler("Response successful!")
            Else
                'call events
                Call EventHandler("Response failed!")
                ''End If
            End If
            '---------------------------------------------------------
        ElseIf cmbCommand.Text = "Initiate Real-Time Streaming" Then
            'call events
            Call EventHandler("Response successful!")

            '---------------------------------------------------------
        ElseIf cmbCommand.Text = "Initiate Low Duty Cycle" Then
            'test for successful response
            If UBound(arrBytes) = 18 And CheckSum(arrBytes) = True Then
                'call events
                Call EventHandler("Response successful!")
                Call EventHandler(vbCrLf + "Just the first Low Duty Cycle packet was trapped and displayed; cycle the power on the node, issue Stop command, etc. to stop Low Duty Cycle.")
            Else
                'call events
                Call EventHandler("Response failed!")
                ''End If
            End If
            '---------------------------------------------------------
        ElseIf cmbCommand.Text = "Initiate Synchronized Sampling" Then
            'test for successful response
            If UBound(arrBytes) = 12 And arrBytes(1) = 7 And CheckSum(arrBytes) = True Then
                'call events
                Call EventHandler("Response successful!")
                Call EventHandler(vbCrLf + "Synchronized Sampling data packets are now being sent by the node; cycle the power on the node, issue Stop command, etc. to stop Synchronized Sampling")
            Else
                'call events
                Call EventHandler("Response failed!")
                ''End If
            End If
            '---------------------------------------------------------
        ElseIf cmbCommand.Text = "Read Single Sensor" Then
            'test for successful response
            If UBound(arrBytes) = 4 And SimpleCheckSum(arrBytes) = True Then
                'call events
                Call EventHandler("Response successful!")
                Call EventHandler(vbCrLf + "Sensor Value: " + Str(CDbl(arrBytes(1) * 256) + CDbl(arrBytes(2))))
            Else
                'call events
                Call EventHandler("Response failed!")
                ''End If
            End If
            '---------------------------------------------------------
        ElseIf cmbCommand.Text = "Auto-Balance Channel" Then
            'There are no response bytes to this command.
            '---------------------------------------------------------
        ElseIf cmbCommand.Text = "Node Discovery" Then
            'pull out the first packet
            Dim arrTemp As Byte()
            ReDim arrTemp(0 To 12)
            Dim MyCounter As Integer
            For MyCounter = 0 To 12
                arrTemp(MyCounter) = arrBytes(MyCounter)
            Next
            'test for successful response
            If UBound(arrBytes) = 25 And (arrBytes(1) And 8) = 0 And CheckSum(arrTemp) = True Then
                'call events
                Call EventHandler("Response successful!")
                Call EventHandler(vbCrLf + "Node Address: " + Str(CDbl(arrBytes(3) * 256) + CDbl(arrBytes(4))))
                Call EventHandler("Radio Channel: " + Str(CDbl(arrBytes(6))))
                Call EventHandler("Model Number: " + Str(CDbl(arrBytes(7) * 256) + CDbl(arrBytes(8))))
                Call EventHandler("LQI: " + Str(arrBytes(9)))
                Call EventHandler("Base Station RSSI: " + Str(CalcRSSI(arrBytes(10))) + " dBm")
            Else
                'call events
                Call EventHandler("Response failed!")
                ''End If
            End If
        End If

        'exit
        Exit Sub
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Sub

    Private Function CheckSum(ByVal arrParse As Byte()) As Boolean
        'dim
        Dim X As Integer, MyCheckSum As Double, TheirCheckSum As Double
        'calc MyCheckSum
        For X = 2 To UBound(arrParse) - 4
            MyCheckSum = MyCheckSum + CDbl(arrParse(X))
        Next
        'calc TheirCheckSum
        TheirCheckSum = CDbl(arrParse(UBound(arrParse) - 1) * 256) + CDbl(arrParse(UBound(arrParse)))
        'test checksums
        If MyCheckSum Mod 65536 = TheirCheckSum Then
            CheckSum = True
        Else
            CheckSum = False
        End If
    End Function

    Private Function Short_Ping() As Boolean
        'set error handler
        On Error GoTo ErrorHandler
        'set serial port number
        SerialPort1.PortName() = "COM" + Trim(Str(numCommPort.Value))
        'set receive threshold
        SerialPort1.ReceivedBytesThreshold = 1
        'open serial port
        SerialPort1.Open()
        'construct command using decimals
        ReDim arrBytes(0 To 2)
        arrBytes(0) = 2
        arrBytes(1) = numNodeAddress.Value \ 256
        arrBytes(2) = numNodeAddress.Value Mod 256
        'send command
        SerialPort1.Write(arrBytes, 0, 3)
        'populate command textbox
        Call ParseArray("Command", arrBytes)
        'exit
        Exit Function
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Function

    Private Function Long_Ping() As Boolean
        'set error handler
        On Error GoTo ErrorHandler
        'set serial port number
        SerialPort1.PortName() = "COM" + Trim(Str(numCommPort.Value))
        'set receive threshold
        SerialPort1.ReceivedBytesThreshold = 13
        'open serial port
        SerialPort1.Open()
        'construct command using decimals
        ReDim arrBytes(0 To 9)
        arrBytes(0) = 170
        arrBytes(1) = 5
        arrBytes(2) = 0
        arrBytes(3) = numNodeAddress.Value \ 256
        arrBytes(4) = numNodeAddress.Value Mod 256
        arrBytes(5) = 2
        arrBytes(6) = 0
        arrBytes(7) = 2
        Dim MyCS As Double, X As Integer
        For X = 1 To 7
            MyCS = MyCS + CDbl(arrBytes(X))
        Next
        arrBytes(8) = MyCS \ 256
        arrBytes(9) = MyCS Mod 256
        'send command
        SerialPort1.Write(arrBytes, 0, 10)
        'populate command textbox
        Call ParseArray("Command", arrBytes)
        'exit
        Exit Function
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Function

    Private Function Read_Node_EEPROM() As Boolean
        'set error handler
        On Error GoTo ErrorHandler
        'set serial port number
        SerialPort1.PortName() = "COM" + Trim(Str(numCommPort.Value))
        'set receive threshold
        SerialPort1.ReceivedBytesThreshold = 13
        'open serial port
        SerialPort1.Open()
        'construct command using decimals
        ReDim arrBytes(0 To 11)
        arrBytes(0) = 170
        arrBytes(1) = 5
        arrBytes(2) = 0
        arrBytes(3) = numNodeAddress.Value \ 256
        arrBytes(4) = numNodeAddress.Value Mod 256
        arrBytes(5) = 4
        arrBytes(6) = 0
        arrBytes(7) = 3
        arrBytes(8) = numEEPROMAddress.Value \ 256
        arrBytes(9) = numEEPROMAddress.Value Mod 256
        Dim MyCS As Double, X As Integer
        For X = 1 To 9
            MyCS = MyCS + CDbl(arrBytes(X))
        Next
        arrBytes(10) = MyCS \ 256
        arrBytes(11) = MyCS Mod 256
        'send command
        SerialPort1.Write(arrBytes, 0, 12)
        'populate command textbox
        Call ParseArray("Command", arrBytes)
        'exit
        Exit Function
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Function
    Private Function Write_Node_EEPROM() As Boolean
        'set error handler
        On Error GoTo ErrorHandler
        'set serial port number
        SerialPort1.PortName() = "COM" + Trim(Str(numCommPort.Value))
        'set receive threshold
        SerialPort1.ReceivedBytesThreshold = 13
        'open serial port
        SerialPort1.Open()
        'construct command using decimals
        ReDim arrBytes(0 To 13)
        arrBytes(0) = 170
        arrBytes(1) = 5
        arrBytes(2) = 0
        arrBytes(3) = numNodeAddress.Value \ 256
        arrBytes(4) = numNodeAddress.Value Mod 256
        arrBytes(5) = 6
        arrBytes(6) = 0
        arrBytes(7) = 4
        arrBytes(8) = numEEPROMAddress.Value \ 256
        arrBytes(9) = numEEPROMAddress.Value Mod 256
        arrBytes(10) = numEEPROMValue.Value \ 256
        arrBytes(11) = numEEPROMValue.Value Mod 256
        Dim MyCS As Double, X As Integer
        For X = 1 To 11
            MyCS = MyCS + CDbl(arrBytes(X))
        Next
        arrBytes(12) = MyCS \ 256
        arrBytes(13) = MyCS Mod 256
        'send command
        SerialPort1.Write(arrBytes, 0, 14)
        'populate command textbox
        Call ParseArray("Command", arrBytes)
        'exit
        Exit Function
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Function

    Private Function Initiate_Sleep_Mode() As Boolean
        'set error handler
        On Error GoTo ErrorHandler
        'set serial port number
        SerialPort1.PortName() = "COM" + Trim(Str(numCommPort.Value))
        'open serial port
        SerialPort1.Open()
        'construct command using decimals
        ReDim arrBytes(0 To 2)
        arrBytes(0) = 50
        arrBytes(1) = numNodeAddress.Value \ 256
        arrBytes(2) = numNodeAddress.Value Mod 256
        'send command
        SerialPort1.Write(arrBytes, 0, 3)
        'populate command textbox
        Call ParseArray("Command", arrBytes)
        'call events
        Call EventHandler("Command was sent!")
        Call EventHandler("There are no response bytes to this command.")
        'exit
        Exit Function
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Function

    Private Function Stop_Node() As Boolean
        'set error handler
        On Error GoTo ErrorHandler
        'set serial port number
        SerialPort1.PortName() = "COM" + Trim(Str(numCommPort.Value))
        'set receive threshold
        SerialPort1.ReceivedBytesThreshold = 3
        'open serial port
        SerialPort1.Open()
        'construct command using decimals
        ReDim arrBytes(0 To 9)
        arrBytes(0) = 170
        arrBytes(1) = 254
        arrBytes(2) = 0
        arrBytes(3) = numNodeAddress.Value \ 256
        arrBytes(4) = numNodeAddress.Value Mod 256
        arrBytes(5) = 2
        arrBytes(6) = 0
        arrBytes(7) = 144
        Dim MyCS As Double, X As Integer
        For X = 1 To 7
            MyCS = MyCS + CDbl(arrBytes(X))
        Next
        arrBytes(8) = MyCS \ 256
        arrBytes(9) = MyCS Mod 256
        'send command
        SerialPort1.Write(arrBytes, 0, 10)
        'populate command textbox
        Call ParseArray("Command", arrBytes)
        'exit
        Exit Function
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Function

    Private Function CalcRSSI(ByVal MyRSSI As Double) As Double
        'calc
        If MyRSSI < 128 Then
            CalcRSSI = MyRSSI
        ElseIf MyRSSI >= 128 Then
            CalcRSSI = (MyRSSI Mod 128) - 128
        End If
    End Function

    Private Function Arm_Node() As Boolean
        'set error handler
        On Error GoTo ErrorHandler
        'set serial port number
        SerialPort1.PortName() = "COM" + Trim(Str(numCommPort.Value))
        'set receive threshold
        SerialPort1.ReceivedBytesThreshold = 14
        'open serial port
        SerialPort1.Open()
        'construct command using decimals
        ReDim arrBytes(0 To 20)
        arrBytes(0) = 170
        arrBytes(1) = 5
        arrBytes(2) = 0
        arrBytes(3) = numNodeAddress.Value \ 256
        arrBytes(4) = numNodeAddress.Value Mod 256
        arrBytes(5) = 13 '2+11=13; Hello World needs 11 bytes
        arrBytes(6) = 0
        arrBytes(7) = 13
        arrBytes(8) = 72 'H
        arrBytes(9) = 101 'e
        arrBytes(10) = 108 'l
        arrBytes(11) = 108 'l
        arrBytes(12) = 111 'o
        arrBytes(13) = 32 'space
        arrBytes(14) = 87 'W
        arrBytes(15) = 111 'o
        arrBytes(16) = 114 'r
        arrBytes(17) = 108 'l
        arrBytes(18) = 100 'd
        Dim MyCS As Double, X As Integer
        For X = 1 To 18
            MyCS = MyCS + CDbl(arrBytes(X))
        Next
        arrBytes(19) = MyCS \ 256
        arrBytes(20) = MyCS Mod 256
        'send command
        SerialPort1.Write(arrBytes, 0, 21)
        'populate command textbox
        Call ParseArray("Command", arrBytes)
        'exit
        Exit Function
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Function
    Private Function Trigger_Armed_Datalogging() As Boolean
        'set error handler
        On Error GoTo ErrorHandler
        'set serial port number
        SerialPort1.PortName() = "COM" + Trim(Str(numCommPort.Value))
        'open serial port
        SerialPort1.Open()
        'construct command using decimals
        ReDim arrBytes(0 To 17)
        arrBytes(0) = 170
        arrBytes(1) = 5
        arrBytes(2) = 0
        arrBytes(3) = numNodeAddress.Value \ 256
        arrBytes(4) = numNodeAddress.Value Mod 256
        arrBytes(5) = 10
        arrBytes(6) = 0
        arrBytes(7) = 14
        arrBytes(8) = 0
        arrBytes(9) = 0
        arrBytes(10) = 0
        arrBytes(11) = 0
        arrBytes(12) = 0
        arrBytes(13) = 0
        arrBytes(14) = 0
        arrBytes(15) = 0
        Dim MyCS As Double, X As Integer
        For X = 1 To 16
            MyCS = MyCS + CDbl(arrBytes(X))
        Next
        arrBytes(16) = MyCS \ 256
        arrBytes(17) = MyCS Mod 256
        'send command
        SerialPort1.Write(arrBytes, 0, 18)
        'populate command textbox
        Call ParseArray("Command", arrBytes)
        'call events
        Call EventHandler("Command was sent!")
        Call EventHandler("There are no response bytes to this command.")
        'exit
        Exit Function
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Function
    Private Function Download_Page() As Boolean
        'set error handler
        On Error GoTo ErrorHandler
        'set serial port number
        SerialPort1.PortName() = "COM" + Trim(Str(numCommPort.Value))
        'set receive threshold
        SerialPort1.ReceivedBytesThreshold = 267
        'open serial port
        SerialPort1.Open()
        'construct command using decimals
        ReDim arrBytes(0 To 4)
        arrBytes(0) = 5
        arrBytes(1) = numNodeAddress.Value \ 256
        arrBytes(2) = numNodeAddress.Value Mod 256
        arrBytes(3) = numPageNumber.Value \ 256
        arrBytes(4) = numPageNumber.Value Mod 256
        'send command
        SerialPort1.Write(arrBytes, 0, 5)
        'populate command textbox
        Call ParseArray("Command", arrBytes)
        'exit
        Exit Function
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Function

    Private Function SimpleCheckSum(ByVal arrParse As Byte()) As Boolean
        'dim
        Dim X As Integer, MyCheckSum As Double, TheirCheckSum As Double
        'calc MyCheckSum
        For X = 1 To UBound(arrParse) - 2
            MyCheckSum = MyCheckSum + CDbl(arrParse(X))
        Next
        'calc TheirCheckSum
        TheirCheckSum = CDbl(arrParse(UBound(arrParse) - 1) * 256) + CDbl(arrParse(UBound(arrParse)))
        'test checksums
        If MyCheckSum Mod 65536 = TheirCheckSum Then
            SimpleCheckSum = True
        Else
            SimpleCheckSum = False
        End If
    End Function

    Private Function MakeDataPoints(ByVal arrbytes) As String
        Dim X As Integer
        'seed var
        MakeDataPoints = vbCrLf + "132 Data Points: "
        For X = 2 To 264 Step 2
            MakeDataPoints = MakeDataPoints + Chr(32) + Str(CDbl(arrbytes(X) * 256) + CDbl(arrbytes(X + 1)))
        Next
    End Function
    Private Function Erase_Session_Data() As Boolean
        'set error handler
        On Error GoTo ErrorHandler
        'set serial port number
        SerialPort1.PortName() = "COM" + Trim(Str(numCommPort.Value))
        'set receive threshold
        SerialPort1.ReceivedBytesThreshold = 1
        'open serial port
        SerialPort1.Open()
        'construct command using decimals
        ReDim arrBytes(0 To 6)
        arrBytes(0) = 6
        arrBytes(1) = numNodeAddress.Value \ 256
        arrBytes(2) = numNodeAddress.Value Mod 256
        arrBytes(3) = 8
        arrBytes(4) = 16
        arrBytes(5) = 12
        arrBytes(6) = 255
        'send command
        SerialPort1.Write(arrBytes, 0, 7)
        'populate command textbox
        Call ParseArray("Command", arrBytes)
        'exit
        Exit Function
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Function
    Private Function Initiate_Real_Time_Streaming() As Boolean
        'set error handler
        On Error GoTo ErrorHandler
        'set serial port number
        SerialPort1.PortName() = "COM" + Trim(Str(numCommPort.Value))
        'set receive threshold
        SerialPort1.ReceivedBytesThreshold = 1020 '1000 sweeps + 10 to 20 garbage bytes
        'open serial port
        SerialPort1.Open()
        'construct command using decimals
        ReDim arrBytes(0 To 2)
        arrBytes(0) = 56
        arrBytes(1) = numNodeAddress.Value \ 256
        arrBytes(2) = numNodeAddress.Value Mod 256
        'send command
        SerialPort1.Write(arrBytes, 0, 3)
        'populate command textbox
        Call ParseArray("Command", arrBytes)
        'exit
        Exit Function
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Function

    Private Sub cmbCommand_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmbCommand.SelectedIndexChanged
        'alert user regarding streaming setup
        If cmbCommand.Text = "Initiate Real-Time Streaming" Then
            txtEvents.Text = "This sample expects that the node has one channel enabled and streaming sweeps have been set to 1000. If the node is configured differently, results will be other than expected."
        ElseIf cmbCommand.Text = "Initiate Low Duty Cycle" Then
            txtEvents.Text = "This sample expects that the node has one channel enabled, that the sampling rate is set to 1 sample every 15 seconds, and that the data type is 0x03. If the node is configured differently, results will be other than expected."
        ElseIf cmbCommand.Text = "Auto-Balance Channel" Then
            txtEvents.Text = "Only applicable to channels 1, 2, 3 and 4 on the V-Link and channel 1 on the SG-Link, SG-Link OEM and DVRT-Link."
        ElseIf cmbCommand.Text = "Node Discovery" Then
            txtEvents.Text = "Turn the node off, click the Send Command button, turn the node on."
        End If
    End Sub
    Private Function Initiate_Low_Duty_Cycle() As Boolean
        'set error handler
        On Error GoTo ErrorHandler
        'set serial port number
        SerialPort1.PortName() = "COM" + Trim(Str(numCommPort.Value))
        'set receive threshold
        SerialPort1.ReceivedBytesThreshold = 19
        'open serial port
        SerialPort1.Open()
        'construct command using decimals
        ReDim arrBytes(0 To 9)
        arrBytes(0) = 170
        arrBytes(1) = 5
        arrBytes(2) = 0
        arrBytes(3) = numNodeAddress.Value \ 256
        arrBytes(4) = numNodeAddress.Value Mod 256
        arrBytes(5) = 2
        arrBytes(6) = 0
        arrBytes(7) = 56
        Dim MyCS As Double, X As Integer
        For X = 1 To 7
            MyCS = MyCS + CDbl(arrBytes(X))
        Next
        arrBytes(8) = MyCS \ 256
        arrBytes(9) = MyCS Mod 256
        'send command
        SerialPort1.Write(arrBytes, 0, 10)
        'populate command textbox
        Call ParseArray("Command", arrBytes)
        'exit
        Exit Function
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Function
    Private Function Read_Single_Sensor() As Boolean
        'set error handler
        On Error GoTo ErrorHandler
        'set serial port number
        SerialPort1.PortName() = "COM" + Trim(Str(numCommPort.Value))
        'set receive threshold
        SerialPort1.ReceivedBytesThreshold = 5
        'open serial port
        SerialPort1.Open()
        'construct command using decimals
        ReDim arrBytes(0 To 4)
        arrBytes(0) = 3
        arrBytes(1) = numNodeAddress.Value \ 256
        arrBytes(2) = numNodeAddress.Value Mod 256
        arrBytes(3) = 1
        arrBytes(4) = numChannelNumber.Value
        'send command
        SerialPort1.Write(arrBytes, 0, 5)
        'populate command textbox
        Call ParseArray("Command", arrBytes)
        'exit
        Exit Function
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Function
    Private Function Auto_Balance_Channel() As Boolean
        'set error handler
        On Error GoTo ErrorHandler
        'set serial port number
        SerialPort1.PortName() = "COM" + Trim(Str(numCommPort.Value))
        'open serial port
        SerialPort1.Open()
        'construct command using decimals
        ReDim arrBytes(0 To 5)
        arrBytes(0) = 98
        arrBytes(1) = numNodeAddress.Value \ 256
        arrBytes(2) = numNodeAddress.Value Mod 256
        arrBytes(3) = numChannelNumber.Value
        arrBytes(4) = 2048 \ 256 'using 2048 mid-scale value
        arrBytes(5) = 2048 Mod 256 'using 2048 mid-scale value
        'send command
        SerialPort1.Write(arrBytes, 0, 6)
        'populate command textbox
        Call ParseArray("Command", arrBytes)
        'call events
        Call EventHandler("Command was sent!")
        Call EventHandler("There are no response bytes to this command.")
        'exit
        Exit Function
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Function

    Private Sub btnClearText_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClearText.Click
        'clear textboxes
        txtEvents.Text = ""
        txtCommandSent.Text = ""
        txtResponseReceived.Text = ""
    End Sub

    Private Function Node_Discovery() As Boolean
        'set error handler
        On Error GoTo ErrorHandler
        'set serial port number
        SerialPort1.PortName() = "COM" + Trim(Str(numCommPort.Value))
        'set receive threshold
        SerialPort1.ReceivedBytesThreshold = 25
        'open serial port
        SerialPort1.Open()
        'call events
        Call EventHandler("Awaiting node power on...")
        'exit
        Exit Function
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Function

    Private Function Initiate_Synchronized_Sampling() As Boolean
        'set error handler
        On Error GoTo ErrorHandler
        'set serial port number
        SerialPort1.PortName() = "COM" + Trim(Str(numCommPort.Value))
        'set receive threshold
        SerialPort1.ReceivedBytesThreshold = 13
        'open serial port
        SerialPort1.Open()
        'construct command using decimals
        ReDim arrBytes(0 To 9)
        arrBytes(0) = 170
        arrBytes(1) = 5
        arrBytes(2) = 0
        arrBytes(3) = numNodeAddress.Value \ 256
        arrBytes(4) = numNodeAddress.Value Mod 256
        arrBytes(5) = 2
        arrBytes(6) = 0
        arrBytes(7) = 59
        Dim MyCS As Double, X As Integer
        For X = 1 To 7
            MyCS = MyCS + CDbl(arrBytes(X))
        Next
        arrBytes(8) = MyCS \ 256
        arrBytes(9) = MyCS Mod 256
        'send command
        SerialPort1.Write(arrBytes, 0, 10)
        'populate command textbox
        Call ParseArray("Command", arrBytes)
        'exit
        Exit Function
ErrorHandler:
        'close serial port
        SerialPort1.Close()
        'call events
        Call EventHandler("Error Message: " + Err.Description)
    End Function

    Private Sub numEEPROMAddress_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles numEEPROMAddress.ValueChanged
        'increment EEPROM address so that it is always an even number
        If (numEEPROMAddress.Value And 1) <> 0 Then
            numEEPROMAddress.Value = 0
        End If
    End Sub
End Class
